
from __future__ import division

import codecs
import itertools

_HEADING = """\
<?xml version="1.0" encoding="UTF-8" standalone="no"?>
<!-- Created with svgtools.py by Facundo Batista -->

<svg
   xmlns:dc="http://purl.org/dc/elements/1.1/"
   xmlns:cc="http://creativecommons.org/ns#"
   xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#"
   xmlns:svg="http://www.w3.org/2000/svg"
   xmlns="http://www.w3.org/2000/svg"
   xmlns:sodipodi="http://sodipodi.sourceforge.net/DTD/sodipodi-0.dtd"
   xmlns:inkscape="http://www.inkscape.org/namespaces/inkscape"
   width="210mm"
   height="297mm"
   id="svg2"
   version="1.1"
   inkscape:version="0.48.0 r9654"
   sodipodi:docname="dibujo.svg">
  <defs
     id="defs4" />
  <sodipodi:namedview
     id="base"
     pagecolor="#ffffff"
     bordercolor="#666666"
     borderopacity="1.0"
     inkscape:pageopacity="0.0"
     inkscape:pageshadow="2"
     inkscape:zoom="1.979899"
     inkscape:cx="160.79774"
     inkscape:cy="833.98977"
     inkscape:document-units="px"
     inkscape:current-layer="layer1"
     showgrid="false"
     inkscape:window-width="1920"
     inkscape:window-height="1005"
     inkscape:window-x="0"
     inkscape:window-y="24"
     inkscape:window-maximized="1" />
  <metadata
     id="metadata7">
    <rdf:RDF>
      <cc:Work
         rdf:about="">
        <dc:format>image/svg+xml</dc:format>
        <dc:type
           rdf:resource="http://purl.org/dc/dcmitype/StillImage" />
        <dc:title>Titulo arbitrario</dc:title>
      </cc:Work>
    </rdf:RDF>
  </metadata>
  <g
     inkscape:label="Capa 1"
     inkscape:groupmode="layer"
     id="layer1">
"""

_FOOTER = """
  </g>
</svg>
"""

_ids = itertools.count()

def get_color(color):
    """Transform different color formats into RRGGBB in hexa."""
    # right now it only checks it's valid
    if len(color) != 6:
        raise ValueError("The color string must be of 6 chars.")
    # check to see if all are hexa
    int(color, 16)
    return color


class Path(object):
    """Path."""

    _style = (
        "fill:#%(color)s;stroke:none;stroke-width:1px;stroke-linecap:butt;"
        "stroke-linejoin:miter;stroke-opacity:1;fill-opacity:0.5"
    )

    def __init__(self, color, points):
        self._id = "path%04d" % _ids.next()
        self._points = points
        self._color = get_color(color)

    def _get_svg_data(self):
        """Return hardcore svg info."""
        points = " ".join(("%s,%s" % p) for p in self._points)
        data = [
            ('style', self._style % dict(color=self._color)),
            ('id', self._id),
            ('d', "m %s z" % points),
        ]
        return data

    def to_svg(self):
        """Renders self to svg."""
        content = "\n  ".join('%s="%s"' % x for x in self._get_svg_data())
        return "<path\n  %s\n/>\n" % content

    def __unicode__(self):
        return u"<svg.Path color=%s points=%s>" % (self._color, self._points)
    __repr__ = __unicode__


class Rect(object):
    """Rectangle."""

    _style = (
        "opacity:1;color:#000000;fill:none;stroke:#000000;stroke-width:2;"
        "stroke-miterlimit:4;stroke-opacity:1;stroke-dasharray:none;"
        "marker:none;visibility:visible;display:inline;overflow:visible;"
        "enable-background:accumulate"
    )

    def __init__(self, x, y, w, h):
        self._id = "rect%04d" % _ids.next()
        self._x = x
        self._y = y
        self._width = w
        self._heigth = h

    def _get_svg_data(self):
        """Return hardcore svg info."""
        data = [
            ('style', self._style),
            ('id', self._id),
            ('width', self._width),
            ('height', self._heigth),
            ('x', self._x),
            ('y', self._y),
            ('rx', 0),
            ('ry', 0),
        ]
        return data

    def to_svg(self):
        """Renders self to svg."""
        content = "\n  ".join('%s="%s"' % x for x in self._get_svg_data())
        return "<rect\n  %s\n/>\n" % content

    def __unicode__(self):
        return u"<svg.Rect (%sx%s) @(%s,%s)>" % (self._width, self._heigth,
                                                self._x, self._y)
    __repr__ = __unicode__


class Text(object):
    """Text."""

    _anchors = ('start', 'middle', 'end')

    _style = (
        "font-size:%spx;font-style:normal;font-variant:normal;"
        "font-weight:normal;font-stretch:normal;text-align:start;"
        "line-height:125%%;letter-spacing:0px;word-spacing:0px;writing-mode:"
        "lr-tb;text-anchor:%s;fill:#000000;fill-opacity:1;stroke:none;"
        "font-family:DejaVu Sans;-inkscape-font-specification:DejaVu Sans"
    )

    def __init__(self, x, y, text, font_size, anchor='start'):
        self._id = "text%04d" % _ids.next()
        self._x = x
        self._y = y
        if isinstance(text, str):
            text = text.decode('utf8')
        self._text = text
        self._font_size = int(font_size)
        if anchor not in self._anchors:
            raise ValueError("Invalid anchor (got %r)" % (anchor,))
        self._anchor = anchor

    def _get_svg_data(self):
        """Return hardcore svg info."""
        data = [
            ('xml:space', 'preserve'),
            ('style', self._style % (self._font_size, self._anchor)),
            ('id', self._id),
            ('x', self._x),
            ('y', self._y),
        ]
        return data

    def to_svg(self):
        """Renders self to svg."""
        content = "\n  ".join('%s="%s"' % x for x in self._get_svg_data())
        return u"<text\n  %s>\n%s\n</text>\n" % (content, self._text)

    def __unicode__(self):
        return u"<svg.Text %dpx @(%s,%s) %r>" % (self._font_size, self._x,
                                                 self._y, self._text)
    __repr__ = __unicode__


class SVG(object):
    """Draw primitives."""

    def __init__(self, title):
        if isinstance(title, str):
            title = title.decode('utf8')
        self.title = title
        self.elements = []

    def add_text(self, *args, **kwargs):
        """Add a Text Element."""
        self.elements.append(Text(*args, **kwargs))

    def add_rect(self, *args, **kwargs):
        """Add a Rect Element."""
        self.elements.append(Rect(*args, **kwargs))

    def add_path(self, *args, **kwargs):
        """Add a Path Element."""
        self.elements.append(Path(*args, **kwargs))

    def save(self, fname):
        """Draw the elements and save to a file."""
        if not self.elements:
            raise ValueError("No elements to save!")

#        # get the margings for first element
#        first = self.elements[0]
#        print "=== first", first
#        left = first.x
#        right = first.x + first.width
#        bottom = first.y
#        top = first.y + first.height
#
#        # max and min for the rest
#        for elem in self.elements[1:]:
#            print "=== elem", elem
#            left = min(left, elem.x)
#            right = max(right, elem.x + elem.width)
#            bottom = min(bottom, elem.y)
#            top = max(top, elem.y + elem.heigh)
#
#        print "==== area:", left, right, bottom, top

        # save to file
        with codecs.open(fname, 'wb', 'utf8') as fh:
#            fh.write(_HEADING % (self.title,))
            fh.write(_HEADING)
            for elem in self.elements:
                fh.write(elem.to_svg())
            fh.write(_FOOTER)

